from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(
                "https://www.gateshead.gov.uk/article/3150/Bin-collection-day-checker"
            )

            accept_button = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.NAME, "acceptall"))
            )
            accept_button.click()

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.ID, "BINCOLLECTIONCHECKER_ADDRESSSEARCH_ADDRESSLOOKUPPOSTCODE")
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "BINCOLLECTIONCHECKER_ADDRESSSEARCH_ADDRESSLOOKUPSEARCH")
                )
            )
            findAddress.click()

            # Wait for the 'Select address' dropdown to appear and select option matching the house name/number
            WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (
                        By.XPATH,
                        "//select[@id='BINCOLLECTIONCHECKER_ADDRESSSEARCH_ADDRESSLOOKUPADDRESS']//option[contains(., '"
                        + user_paon
                        + "')]",
                    )
                )
            ).click()

            # Wait for the collections table to appear
            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, ".bincollections__table")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # Get collections table
            table = soup.find("table", {"class": "bincollections__table"})

            # Get rows
            month_year = ""
            for row in table.find_all("tr"):
                if row.find("th"):
                    month_year = (
                        row.find("th").get_text(strip=True)
                        + " "
                        + datetime.now().strftime("%Y")
                    )
                elif month_year != "":
                    collection = row.find_all("td")
                    bin_date = datetime.strptime(
                        collection[0].get_text(strip=True) + " " + month_year,
                        "%d %B %Y",
                    )
                    dict_data = {
                        "type": collection[2]
                        .get_text()
                        .replace("- DAY CHANGE", "")
                        .strip(),
                        "collectionDate": bin_date.strftime(date_format),
                    }
                    data["bins"].append(dict_data)

            data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
            )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from datetime import datetime
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):

    def parse_data(self, page: str, **kwargs) -> dict:
        print(f"Arguments are f{kwargs}")
        driver = None
        try:
            page = kwargs["url"]
            street_name = kwargs.get("paon")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            wait = WebDriverWait(driver, 60)

            self.dismiss_cookie_banner(wait)
            self.input_street_name(street_name, wait)
            self.submit(wait)
            bin_types, collection_days = self.get_bins(driver)
            bindata = self.get_collection_days(bin_types, collection_days)

            print(bindata)

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return bindata

    def get_collection_days(self, bin_types, collection_days):
        bindata = {"bins": []}
        WEEKLY_COLLECTION = 0
        GARDEN_COLLECTION = 1

        for index, bin_type in enumerate(bin_types):
            # currently only handled weekly and garden collection, special collections like Christmas Day need to be added
            if index == WEEKLY_COLLECTION:
                next_collection_date = get_next_day_of_week(
                    collection_days[index].text.strip(), date_format
                )
            elif index == GARDEN_COLLECTION:
                split_date_part = collection_days[index].text.split("More dates")[0]
                next_collection_date = datetime.strptime(
                    split_date_part.strip(), "%d %B %Y"
                ).strftime(date_format)
            else:
                next_collection_date = datetime.strptime(
                    collection_days[index].text.strip(), "%d %B %Y"
                ).strftime(date_format)

            dict_data = {
                "type": bin_type.text.strip(),
                "collectionDate": next_collection_date,
            }
            bindata["bins"].append(dict_data)
        return bindata

    def get_bins(self, driver):
        table = driver.find_element(By.XPATH, ".//div[@id='maincontent']//table")
        table_rows = table.find_elements(by=By.TAG_NAME, value="tr")
        headerRow = table_rows[0]
        table_info_row = table_rows[1]
        bin_types = headerRow.find_elements(by=By.TAG_NAME, value="th")[2:]
        collection_days = table_info_row.find_elements(by=By.TAG_NAME, value="td")[2:]
        return bin_types, collection_days

    def submit(self, wait):
        main_content_submit_button = wait.until(
            EC.element_to_be_clickable(
                (By.XPATH, ".//div[@id='maincontent']//input[@type='submit']")
            )
        )
        main_content_submit_button.send_keys(Keys.ENTER)

    def input_street_name(self, street_name, wait):
        input_element_postcodesearch = wait.until(
            EC.visibility_of_element_located((By.ID, "Street"))
        )
        input_element_postcodesearch.send_keys(street_name)

    def dismiss_cookie_banner(self, wait):
        cookie_banner = wait.until(
            EC.visibility_of_element_located((By.ID, "ccc-dismiss-button"))
        )
        cookie_banner.send_keys(Keys.ENTER)



from datetime import datetime, timedelta

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://www.cherwell.gov.uk/homepage/129/bin-collection-search?uprn={user_uprn}"

        # Make the GET request
        response = requests.get(URI)

        soup = BeautifulSoup(response.text, "html.parser")

        def get_full_date(date_str):
            # Get the current year
            current_year = datetime.today().year

            date_str = remove_ordinal_indicator_from_date_string(date_str)

            # Convert the input string to a datetime object (assuming the current year first)
            date_obj = datetime.strptime(f"{date_str} {current_year}", "%d %B %Y")

            # If the date has already passed this year, use next year
            if date_obj < datetime.today():
                date_obj = datetime.strptime(
                    f"{date_str} {current_year + 1}", "%d %B %Y"
                )

            return date_obj.strftime(date_format)  # Return in YYYY-MM-DD format

        # print(soup)

        div = soup.find("div", class_="bin-collection-results__tasks")

        for item in div.find_all("li", class_="list__item"):
            # Extract bin type
            bin_type_tag = item.find("h3", class_="bin-collection-tasks__heading")
            bin_type = (
                "".join(bin_type_tag.find_all(text=True, recursive=False)).strip()
                if bin_type_tag
                else "Unknown Bin"
            )

            # Extract collection date
            date_tag = item.find("p", class_="bin-collection-tasks__date")
            collection_date = date_tag.text.strip() if date_tag else "Unknown Date"

            dict_data = {
                "type": bin_type,
                "collectionDate": get_full_date(collection_date),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return bindata



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_date(self, date_str):
        date_formats = [
            "This %A - %d %b %Y",  # Changed %B to %b to accommodate abbreviated month names
            "Next %A - %d %b %Y",  # Same change here
            "%A %d %b %Y",  # And here
        ]
        for format in date_formats:
            try:
                return datetime.strptime(date_str, format).strftime("%d/%m/%Y")
            except ValueError:
                continue
        raise ValueError(f"Date format not recognized: {date_str}")

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            # Make a BS4 object

            page = "https://www.hounslow.gov.uk/info/20272/recycling_and_waste_collection_day_finder"

            user_postcode = kwargs.get("postcode")
            user_uprn = kwargs.get("uprn")
            user_paon = kwargs.get("paon")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            wait = WebDriverWait(driver, 60)

            inputElement_postcodesearch = wait.until(
                EC.element_to_be_clickable((By.ID, "Postcode"))
            )

            inputElement_postcodesearch.send_keys(user_postcode)

            inputElement_postcodesearch_btn = wait.until(
                EC.element_to_be_clickable((By.ID, "findAddress"))
            )
            inputElement_postcodesearch_btn.click()

            inputElement_select_address = wait.until(
                EC.element_to_be_clickable((By.ID, "UPRN"))
            )

            select_element = wait.until(
                EC.visibility_of_element_located((By.ID, "UPRN"))
            )  # Adjust this ID to your element's ID

            # Create a Select object
            select = Select(select_element)

            # Fetch all options
            options = select.options

            # Loop through options to find the one that starts with the UPRN
            for option in options:
                if option.get_attribute("value").startswith(f"{user_uprn}|"):
                    option.click()  # Select the matching option
                    break

            results = wait.until(
                EC.element_to_be_clickable((By.CLASS_NAME, "bin_day_main_wrapper"))
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            # Find all headers which include collection dates
            collection_headers = soup.find_all("h4")
            bins_data = []

            # Process each collection date and corresponding bins
            for header in collection_headers:
                date_text = header.get_text(strip=True)
                collection_date = self.parse_date(date_text)

                # Get next sibling which should be the list of bins
                bin_list = header.find_next_sibling("ul")
                if bin_list:
                    for item in bin_list.find_all("li", class_="list-group-item"):
                        bin_type = item.get_text(strip=True)
                        bins_data.append(
                            {"type": bin_type, "collectionDate": collection_date}
                        )

            # Construct the final JSON object
            json_data = {"bins": bins_data}

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return json_data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        # UPRN is passed in via a cookie. Set cookies/params and GET the page
        cookies = {
            # 'JSESSIONID': '96F2A15C14569B2ED2BBEB140FE86532',
            "SVBINZONE": f"SOUTH%3AUPRN%40{user_uprn}",
        }
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8",
            "Accept-Language": "en-GB,en;q=0.7",
            "Cache-Control": "max-age=0",
            "Connection": "keep-alive",
            "Referer": "https://eform.southoxon.gov.uk/ebase/BINZONE_DESKTOP.eb?SOVA_TAG=SOUTH&ebd=0&ebz=1_1668467255368",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-User": "?1",
            "Sec-GPC": "1",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
        }
        params = {
            "SOVA_TAG": "SOUTH",
            "ebd": "0",
            # 'ebz':      '1_1668467255368',
        }
        requests.packages.urllib3.disable_warnings()
        response = requests.get(
            "https://eform.southoxon.gov.uk/ebase/BINZONE_DESKTOP.eb",
            params=params,
            headers=headers,
            cookies=cookies,
        )

        # Parse response text for super speedy finding
        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        current_year = datetime.now().year
        next_year = current_year + 1

        # Page has slider info side by side, which are two instances of this class
        for bin in soup.find_all("div", {"class": "binextra"}):
            bin_info = list(bin.stripped_strings)
            try:
                # On standard collection schedule, date will be contained in the first stripped string
                if contains_date(bin_info[0]):
                    bin_date = get_next_occurrence_from_day_month(
                        datetime.strptime(
                            bin_info[0],
                            "%A %d %B -",
                        )
                    )
                    bin_type = str.capitalize(" ".join(bin_info[1:]))
                # On exceptional collection schedule (e.g. around English Bank Holidays), date will be contained in the second stripped string
                else:
                    bin_date = get_next_occurrence_from_day_month(
                        datetime.strptime(
                            bin_info[1],
                            "%A %d %B -",
                        )
                    )
                    bin_type = str.capitalize(" ".join(bin_info[2:]))
            except:
                continue

            if (datetime.now().month == 12) and (bin_date.month == 1):
                bin_date = bin_date.replace(year=next_year)
            else:
                bin_date = bin_date.replace(year=current_year)

            # Build data dict for each entry
            dict_data = {
                "type": bin_type,
                "collectionDate": bin_date.strftime(date_format),
            }
            data["bins"].append(dict_data)

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        user_uprn = user_uprn.zfill(12)
        bindata = {"bins": []}

        URI = "https://www.west-norfolk.gov.uk/info/20174/bins_and_recycling_collection_dates"

        headers = {"Cookie": f"bcklwn_uprn={user_uprn}"}

        # Make the GET request
        response = requests.get(URI, headers=headers)

        soup = BeautifulSoup(response.content, features="html.parser")
        soup.prettify()

        # Find all bin_date_container divs
        bin_date_containers = soup.find_all("div", class_="bin_date_container")

        # Loop through each bin_date_container
        for container in bin_date_containers:
            # Extract the collection date
            date = (
                container.find("h3", class_="collectiondate").text.strip().rstrip(":")
            )

            # Extract the bin type from the alt attribute of the img tag
            bin_type = container.find("img")["alt"]

            dict_data = {
                "type": bin_type,
                "collectionDate": datetime.strptime(
                    date,
                    "%A %d %B %Y",
                ).strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")
        check_postcode(user_postcode)
        check_paon(user_paon)
        bindata = {"bins": []}

        user_postcode = user_postcode.replace(" ", "+")

        URI = f"https://www.royalgreenwich.gov.uk/site/custom_scripts/apps/waste-collection/new2023/source.php?term={user_postcode}"

        # Make the GET request
        response = requests.get(URI)

        for address in response.json():
            if user_paon in address:
                collection_address = address
                break

        URI = "https://www.royalgreenwich.gov.uk/site/custom_scripts/repo/apps/waste-collection/new2023/ajax-response-uprn.php"

        data = {"address": collection_address}

        response = requests.post(URI, data=data)

        response = response.json()

        collection_day = response["Day"]
        week = response["Frequency"]

        days_of_week = [
            "Monday",
            "Tuesday",
            "Wednesday",
            "Thursday",
            "Friday",
            "Saturday",
            "Sunday",
        ]
        collectionweek = ["Week A", "Week B"]

        offset_days = days_of_week.index(collection_day)
        week = collectionweek.index(week)

        greenstartDate = datetime(2024, 11, 25)
        bluestartDate = datetime(2024, 11, 25)
        if week == 0:
            blackstartDate = datetime(2024, 11, 18)
        elif week == 1:
            blackstartDate = datetime(2024, 11, 25)

        green_dates = get_dates_every_x_days(greenstartDate, 7, 100)
        blue_dates = get_dates_every_x_days(bluestartDate, 7, 100)
        black_dates = get_dates_every_x_days(blackstartDate, 14, 50)

        for greenDate in green_dates:

            collection_date = (
                datetime.strptime(greenDate, "%d/%m/%Y") + timedelta(days=offset_days)
            ).strftime("%d/%m/%Y")

            dict_data = {
                "type": "Green Bin",
                "collectionDate": collection_date,
            }
            bindata["bins"].append(dict_data)

        for blueDate in blue_dates:

            collection_date = (
                datetime.strptime(blueDate, "%d/%m/%Y") + timedelta(days=offset_days)
            ).strftime("%d/%m/%Y")

            dict_data = {
                "type": "Blue Bin",
                "collectionDate": collection_date,
            }
            bindata["bins"].append(dict_data)

        for blackDate in black_dates:

            collection_date = (
                datetime.strptime(blackDate, "%d/%m/%Y") + timedelta(days=offset_days)
            ).strftime("%d/%m/%Y")

            dict_data = {
                "type": "Black Bin",
                "collectionDate": collection_date,
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = "https://www.breckland.gov.uk/apiserver/ajaxlibrary"

        data = {
            "id": "1730410741649",
            "jsonrpc": "2.0",
            "method": "Breckland.Whitespace.JointWasteAPI.GetBinCollectionsByUprn",
            "params": {"uprn": user_uprn, "environment": "live"},
        }
        # Make the GET request
        response = requests.post(URI, json=data)

        # Parse the JSON response
        bin_collection = response.json()

        # Loop through each collection in bin_collection
        for collection in bin_collection["result"]:
            bin_type = collection.get("collectiontype")
            collection_date = collection.get("nextcollection")

            dict_data = {
                "type": bin_type,
                "collectionDate": datetime.strptime(
                    collection_date,
                    "%d/%m/%Y %H:%M:%S",
                ).strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://online.aberdeenshire.gov.uk/Apps/Waste-Collections/Routes/Route/{user_uprn}"

        # Make the GET request
        response = requests.get(URI)

        soup = BeautifulSoup(response.content, features="html.parser")
        soup.prettify()

        for collection in soup.find("table").find("tbody").find_all("tr"):
            th = collection.find("th")
            if th:
                continue
            td = collection.find_all("td")
            collection_date = datetime.strptime(
                td[0].text,
                "%d/%m/%Y %A",
            )
            bin_type = td[1].text.split(" and ")

            for bin in bin_type:
                dict_data = {
                    "type": bin,
                    "collectionDate": collection_date.strftime(date_format),
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get("https://www.boston.gov.uk/findwastecollections")

            accept_button = WebDriverWait(driver, timeout=30).until(
                EC.element_to_be_clickable((By.NAME, "acceptall"))
            )
            accept_button.click()

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.ID, "BBCWASTECOLLECTIONS_START_SEARCHPOSTCODE")
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "BBCWASTECOLLECTIONS_START_START10_NEXT")
                )
            )
            findAddress.click()

            # Wait for the custom dropdown container to be visible
            WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (By.ID, "BBCWASTECOLLECTIONS_COLLECTIONADDRESS_INCIDENTUPRN_chosen")
                )
            )

            # Click on the dropdown to open it
            dropdown = driver.find_element(
                By.ID, "BBCWASTECOLLECTIONS_COLLECTIONADDRESS_INCIDENTUPRN_chosen"
            )
            dropdown.click()

            # Wait for the dropdown options to be visible
            WebDriverWait(driver, 10).until(
                EC.visibility_of_element_located((By.CLASS_NAME, "chosen-results"))
            )

            # Locate the desired option using its text
            desired_option = driver.find_element(
                By.XPATH,
                "//li[@class='active-result' and contains(text(), '"
                + user_paon
                + "')]",
            )

            # Click on the desired option
            desired_option.click()

            # dropdown.select_by_visible_text(user_paon)

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "BBCWASTECOLLECTIONS_COLLECTIONADDRESS_NEXT3_NEXT")
                )
            )
            findAddress.click()

            # Wait for the collections table to appear
            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "BBCWASTECOLLECTIONS_SERVICE_FIELD859_OUTER")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # Find the container with the bin information
            bins = soup.find_all(
                "div", class_="grid__cell grid__cell--listitem grid__cell--cols1"
            )

            current_year = datetime.now().year
            next_year = current_year + 1

            # Loop through each bin container to extract the details
            for bin_div in bins:
                # Find the bin type (title text)
                bin_type = bin_div.find("h2", class_="item__title").text.strip()

                # Find the next collection date
                next_collection = (
                    bin_div.find("div", class_="item__content")
                    .find("div")
                    .text.strip()
                    .replace("Next: ", "")
                )

                next_collection = datetime.strptime(
                    remove_ordinal_indicator_from_date_string(next_collection),
                    "%A %d %B",
                )

                if (datetime.now().month == 12) and (next_collection.month == 1):
                    next_collection = next_collection.replace(year=next_year)
                else:
                    next_collection = next_collection.replace(year=current_year)

                dict_data = {
                    "type": bin_type,
                    "collectionDate": next_collection.strftime(date_format),
                }
                data["bins"].append(dict_data)

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = "https://refusecalendarapi.denbighshire.gov.uk/Csrf/token"

        token = requests.get(URI)

        token_data = token.json()

        URI = f"https://refusecalendarapi.denbighshire.gov.uk/Calendar/{user_uprn}"

        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
            "content-type": "application/json",
            "dnt": "1",
            "host": "refusecalendarapi.denbighshire.gov.uk",
            "referer": "https://refusecalendarapi.denbighshire.gov.uk/",
            "x-csrf-token": token_data["token"],
        }

        # Make the GET request
        response = requests.get(URI, headers=headers)

        # Parse the JSON response
        json_data = response.json()

        bin_types = {
            "refuseDate": "Refuse",
            "recyclingDate": "Recycling",
            "gardenDate": "Garden Waste",
            "ahpDate": "AHP (Assisted Household Pickup)",
            "tradeDate": "Trade Waste",
            "tradeRefuseDate": "Trade Refuse",
            "tradeRecyclingDate": "Trade Recycling",
        }

        bindata["bins"] = [
            {"type": label, "collectionDate": json_data[key]}
            for key, label in bin_types.items()
            if json_data.get(key)
        ]

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from datetime import datetime

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)
        bindata = {"bins": []}

        API_URL = "https://www.broxbourne.gov.uk/xfp/form/205"

        post_data = {
            "page": "490",
            "locale": "en_GB",
            "qacf7e570cf99fae4cb3a2e14d5a75fd0d6561058_0_0": user_postcode,
            "qacf7e570cf99fae4cb3a2e14d5a75fd0d6561058_1_0": user_uprn,
            "next": "Next",
        }

        r = requests.post(API_URL, data=post_data)
        r.raise_for_status()

        soup = BeautifulSoup(r.content, features="html.parser")
        soup.prettify()

        form__instructions = soup.find(attrs={"class": "form__instructions"})
        table = form__instructions.find("table")

        rows = table.find_all("tr")

        current_year = datetime.now().year
        current_month = datetime.now().month

        # Process each row into a list of dictionaries
        for row in rows[1:]:  # Skip the header row
            columns = row.find_all("td")
            collection_date_text = (
                columns[0].get_text(separator=" ").replace("\xa0", " ").strip()
            )
            service = columns[1].get_text(separator=" ").replace("\xa0", " ").strip()

            # Safely try to parse collection date
            if collection_date_text:
                try:
                    collection_date = datetime.strptime(
                        collection_date_text, "%a %d %b"
                    )
                    if collection_date.month == 1 and current_month != 1:
                        collection_date = collection_date.replace(year=current_year + 1)
                    else:
                        collection_date = collection_date.replace(year=current_year)

                    formatted_collection_date = collection_date.strftime(
                        "%d/%m/%Y"
                    )  # Use your desired date format
                    dict_data = {
                        "type": service,
                        "collectionDate": formatted_collection_date,
                    }
                    bindata["bins"].append(dict_data)
                except ValueError:
                    # Skip invalid collection_date
                    continue

        # Sort valid bins by collectionDate
        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )
        return bindata



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        collections = []
        data = {"bins": []}

        # Get and check postcode and PAON
        postcode = kwargs.get("postcode")
        paon = kwargs.get("paon")
        check_postcode(postcode)
        check_paon(paon)

        # Make API call to get property info using postcode
        addr_response = requests.get(
            f'https://www.bury.gov.uk/app-services/getProperties?postcode={postcode.replace(" ", "")}'
        )
        if addr_response.status_code != 200:
            raise ConnectionAbortedError("Issue encountered getting addresses.")
        address_json = json.loads(addr_response.text)["response"]

        # This makes addr the next item that has the house number. Since these are ordered by house number, a single
        # number like 3 wouldn't return 33
        addr = next(item for item in address_json if paon in item["addressLine1"])

        # Make API call to get bin data using property ID
        response = requests.get(
            f'https://www.bury.gov.uk/app-services/getPropertyById?id={addr.get("id")}'
        )
        if response.status_code != 200:
            raise ConnectionAbortedError("Issue encountered getting bin data.")
        bin_list = json.loads(response.text)["response"]["bins"]

        # The JSON actually returns the next collections and a large calendar. But I opted just for the next dates.
        for bin_colour, collection_data in bin_list.items():
            bin_type = bin_colour
            bin_date = datetime.strptime(
                remove_ordinal_indicator_from_date_string(
                    collection_data.get("nextCollection")
                ),
                "%A %d %B %Y",
            )
            collections.append((bin_type, bin_date))

        # Dates are ordered correctly - soonest first
        ordered_data = sorted(collections, key=lambda x: x[1])
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(
                "https://iportal.itouchvision.com/icollectionday/collection-day/?uuid=238D5F9796C12643D190E3505931401A8C003F0D&lang=en"
            )

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "postcodeSearch"))
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.XPATH, '//button[@class="govuk-button mt-4"]')
                )
            )
            findAddress.click()

            # Wait for the dropdown to be visible
            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "addressSelect"))
            )

            dropdown = Select(driver.find_element(By.ID, "addressSelect"))
            dropdown.select_by_value(user_uprn)

            # Wait for the collections table to appear
            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        '//div[@class="ant-row d-flex justify-content-between mb-4 mt-2 css-2rgkd4"]',
                    )
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            recyclingcalendar = soup.find(
                "div",
                {
                    "class": "ant-row d-flex justify-content-between mb-4 mt-2 css-2rgkd4"
                },
            )

            rows = recyclingcalendar.find_all(
                "div",
                {
                    "class": "ant-col ant-col-xs-12 ant-col-sm-12 ant-col-md-12 ant-col-lg-12 ant-col-xl-12 css-2rgkd4"
                },
            )

            current_year = datetime.now().year
            current_month = datetime.now().month

            for row in rows:
                BinType = row.find("h3").text
                collectiondate = datetime.strptime(
                    row.find("div", {"class": "text-white fw-bold"}).text,
                    "%A %d %B",
                )
                if (current_month > 10) and (collectiondate.month < 3):
                    collectiondate = collectiondate.replace(year=(current_year + 1))
                else:
                    collectiondate = collectiondate.replace(year=current_year)

                dict_data = {
                    "type": BinType,
                    "collectionDate": collectiondate.strftime("%d/%m/%Y"),
                }
                data["bins"].append(dict_data)

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        SESSION_URL = "https://pkc-self.achieveservice.com/authapi/isauthenticated?uri=https%253A%252F%252Fpkc-self.achieveservice.com%252Fen%252FAchieveForms%252F%253Fform_uri%253Dsandbox-publish%253A%252F%252FAF-Process-de9223b1-a7c6-408f-aaa3-aee33fd7f7fa%252FAF-Stage-9fa33e2e-4c1b-4963-babf-4348ab8154bc%252Fdefinition.json%2526redirectlink%253D%25252Fen%2526cancelRedirectLink%253D%25252Fen%2526consentMessage%253Dyes&hostname=pkc-self.achieveservice.com&withCredentials=true"

        API_URL = "https://pkc-self.achieveservice.com/apibroker/runLookup"

        data = {
            "formValues": {
                "Bin collections": {"propertyUPRNQuery": {"value": user_uprn}}
            },
        }

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "Mozilla/5.0",
            "X-Requested-With": "XMLHttpRequest",
            "Referer": "https://pkc-self.achieveservice.com/fillform/?iframe_id=fillform-frame-1&db_id=",
        }
        s = requests.session()
        r = s.get(SESSION_URL)
        r.raise_for_status()
        session_data = r.json()
        sid = session_data["auth-session"]
        params = {
            "id": "5c9267cee5efe",
            "repeat_against": "",
            "noRetry": "true",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }

        r = s.post(API_URL, json=data, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()
        rows_data = data["integration"]["transformed"]["rows_data"]["0"]
        if not isinstance(rows_data, dict):
            raise ValueError("Invalid data returned from API")

        schedule = {
            "Green Bin": [
                rows_data.get("nextGeneralWasteCollectionDate"),
                rows_data.get("nextGeneralWasteCollectionDate2nd"),
            ],
            "Blue Bin": [
                rows_data.get("nextBlueCollectionDate"),
                rows_data.get("nextBlueWasteCollectionDate2nd"),
            ],
            "Grey Bin": [
                rows_data.get("nextGreyWasteCollectionDate"),
                rows_data.get("nextGreyWasteCollectionDate2nd"),
            ],
            "Brown Bin": [
                rows_data.get("nextGardenandFoodWasteCollectionDate"),
                rows_data.get("nextGardenandFoodWasteCollectionDate2nd"),
            ],
            "Paper Waste": [
                rows_data.get("nextPaperWasteCollectionDate"),
                rows_data.get("nextPaperWasteCollectionDate2nd"),
            ],
        }

        # Format and output the schedule
        for bin_type, dates in schedule.items():
            if any(dates):
                for date in dates:
                    dict_data = {"type": bin_type, "collectionDate": date}
                    bindata["bins"].append(dict_data)

        return bindata



from datetime import datetime

import requests
from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get and check UPRN
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        uri = "https://waste.southhams.gov.uk/mycollections"

        s = requests.session()
        r = s.get(uri)
        for cookie in r.cookies:
            if cookie.name == "fcc_session_cookie":
                fcc_session_token = cookie.value

        uri = "https://waste.southhams.gov.uk/mycollections/getcollectiondetails"

        params = {
            "fcc_session_token": fcc_session_token,
            "uprn": user_uprn,
        }

        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36",
            "Referer": "https://waste.southhams.gov.uk/mycollections",
            "X-Requested-With": "XMLHttpRequest",
        }

        # Send a POST request with form data and headers
        r = s.post(uri, data=params, headers=headers)

        result = r.json()

        for collection in result["binCollections"]["tile"]:

            # Parse the HTML with BeautifulSoup
            soup = BeautifulSoup(collection[0], "html.parser")
            soup.prettify()

            # Find all collectionDiv elements
            collections = soup.find_all("div", class_="collectionDiv")

            # Process each collectionDiv
            for collection in collections:
                # Extract the service name
                service_name = collection.find("h3").text.strip()

                # Extract collection frequency and day
                details = collection.find("div", class_="detWrap").text.strip()

                # Extract the next collection date
                next_collection = details.split("Your next scheduled collection is ")[
                    1
                ].split(".")[0]

                if next_collection.startswith("today"):
                    next_collection = next_collection.split("today, ")[1]
                elif next_collection.startswith("tomorrow"):
                    next_collection = next_collection.split("tomorrow, ")[1]

                dict_data = {
                    "type": service_name,
                    "collectionDate": datetime.strptime(
                        next_collection, "%A, %d %B %Y"
                    ).strftime(date_format),
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import re
import time

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select, WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        web_driver = kwargs.get("web_driver")
        headless = kwargs.get("headless")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        # Initialize the WebDriver (Chrome in this case)
        with create_webdriver(
            web_driver,
            headless,
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36",
            __name__,
        ) as driver:

            # Step 1: Navigate to the form page
            driver.get(
                "https://lewisham.gov.uk/myservices/recycling-and-rubbish/your-bins/collection"
            )

            try:
                cookie_accept_button = WebDriverWait(driver, 5).until(
                    EC.element_to_be_clickable(
                        (By.ID, "CybotCookiebotDialogBodyLevelButtonLevelOptinAllowAll")
                    )
                )
                cookie_accept_button.click()
            except Exception:
                print("No cookie consent banner found or already dismissed.")

            # Wait for the form to load
            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.CLASS_NAME, "address-finder"))
            )

            # Step 2: Locate the input field for the postcode
            postcode_input = driver.find_element(
                By.CLASS_NAME, "js-address-finder-input"
            )

            # Enter the postcode
            postcode_input.send_keys(
                user_postcode
            )  # Replace with your desired postcode
            time.sleep(1)  # Optional: Wait for the UI to react

            # Step 4: Click the "Find address" button with retry logic
            find_button = WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (By.CLASS_NAME, "js-address-finder-step-address")
                )
            )
            find_button.click()

            # Wait for the address selector to appear and options to load
            WebDriverWait(driver, 10).until(
                lambda d: len(
                    d.find_element(By.ID, "address-selector").find_elements(
                        By.TAG_NAME, "option"
                    )
                )
                > 1
            )

            # Select the dropdown and print available options
            address_selector = driver.find_element(By.ID, "address-selector")

            # Use Select class to interact with the dropdown
            select = Select(address_selector)
            if len(select.options) > 1:
                select.select_by_value(user_uprn)
            else:
                print("No additional addresses available to select")

            # Wait until the URL contains the expected substring
            WebDriverWait(driver, 10).until(
                EC.url_contains("/find-your-collection-day-result")
            )

            # Parse the HTML
            soup = BeautifulSoup(driver.page_source, "html.parser")

            # Extract the main container
            collection_result = soup.find("div", class_="js-find-collection-result")

            # Extract each collection type and its frequency/day
            for strong_tag in collection_result.find_all("strong"):
                bin_type = strong_tag.text.strip()  # e.g., "Food waste"
                # Extract the sibling text
                schedule_text = (
                    strong_tag.next_sibling.next_sibling.next_sibling.text.strip()
                    .replace("\n", " ")
                    .replace("\t", " ")
                )

                # Extract the day using regex
                print(schedule_text)
                day_match = re.search(r"on\s*(\w+day)", schedule_text)
                print(day_match)
                day = day_match.group(1) if day_match else None

                # Extract the next collection date using regex
                date_match = re.search(
                    r"Your next collection date is\s*(\d{2}/\d{2}/\d{4})(.?)",
                    schedule_text,
                )
                if date_match:
                    next_collection_date = date_match.group(1)
                else:
                    next_collection_date = get_next_day_of_week(day, date_format)

                dict_data = {
                    "type": bin_type,
                    "collectionDate": next_collection_date,
                }
                bindata["bins"].append(dict_data)

            return bindata



import json
from datetime import timedelta

import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get and check both the passed UPRN and postcode
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)

        # Build the headers, specify the parameters and then make a GET for the calendar
        headers = {
            "Connection": "Keep-Alive",
            "User-Agent": "okhttp/3.14.9",
        }
        params = {
            "end_date": "2024-01-01",
            "rn": user_uprn,
            "device": "undefined",
            "postcode": user_postcode,
            "OS": "android",
            "OS_ver": "31",
            "app_ver": "35",
        }
        requests.packages.urllib3.disable_warnings()
        response = requests.get(
            "http://www.sac-bins.co.uk/get_calendar.php", params=params, headers=headers
        )

        # Load the response as JSON
        json_data = json.loads(response.text)

        # The response loads well over a year's worth of data, so figure out some dates to limit output
        today = datetime.today()
        eight_weeks = datetime.today() + timedelta(days=8 * 7)
        data = {"bins": []}

        # The bin titles are pretty weird and colours are too basic, so make the names match the app
        bin_friendly_names = {
            "blue": "Blue Bin",
            "red": "Food Caddy",
            "green": "Green Bin",
            "grey": "Grey Bin",
            "purple": "Purple Bin",
            "brown": "Brown Bin",
        }

        # Loop through the results. When a date is found that's on or greater than today's date AND less than
        # eight weeks away, we want it in the output. So look up the friendly name and add it in.
        for item in json_data:
            bin_date = datetime.strptime(item["start"], "%Y-%m-%d").date()
            if today.date() <= bin_date <= eight_weeks.date():
                bin_type = bin_friendly_names.get(item["className"])
                dict_data = {
                    "type": bin_type,
                    "collectionDate": bin_date.strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data



import math
from datetime import *

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        user_postcode = kwargs.get("postcode")
        check_uprn(user_uprn)
        check_postcode(user_postcode)

        requests.packages.urllib3.disable_warnings()
        s = requests.Session()

        # Get the first form
        response = s.get(
            "https://my.northtyneside.gov.uk/category/81/bin-collection-dates",
            verify=False,
        )

        # Find the form ID and submit with a postcode
        soup = BeautifulSoup(response.text, features="html.parser")
        form_build_id = soup.find("input", {"name": "form_build_id"})["value"]
        response = s.post(
            "https://my.northtyneside.gov.uk/category/81/bin-collection-dates",
            data={
                "postcode": user_postcode,
                "op": "Find",
                "form_build_id": form_build_id,
                "form_id": "ntc_address_wizard",
            },
            verify=False,
        )

        # Find the form ID and submit with the UPRN
        soup = BeautifulSoup(response.text, features="html.parser")
        form_build_id = soup.find("input", {"name": "form_build_id"})["value"]
        response = s.post(
            "https://my.northtyneside.gov.uk/category/81/bin-collection-dates",
            data={
                "house_number": f"0000{user_uprn}",
                "op": "Use",
                "form_build_id": form_build_id,
                "form_id": "ntc_address_wizard",
            },
            verify=False,
        )

        # Parse form page and get the day of week and week offsets
        soup = BeautifulSoup(response.text, features="html.parser")
        info_section = soup.find("section", {"class": "block block-ntc-bins clearfix"})

        regular_day, garden_day, special_day = None, None, None
        # Get day of week and week label for refuse, garden and special collections.
        # Week label is A or B.  Convert that to an int to use as an offset.
        for anchor in info_section.findAll("a"):
            if anchor.text.startswith("Refuse and Recycling"):
                regular_day = anchor.text.strip().split()[-3]
                if anchor.text.strip().split()[-1] == "A":
                    regular_week = 0
                else:
                    regular_week = 1
            elif anchor.text.startswith("Garden Waste"):
                garden_day = anchor.text.strip().split()[-3]
                if anchor.text.strip().split()[-1] == "A":
                    garden_week = 0
                else:
                    garden_week = 1
        for para in info_section.findAll("p"):
            if para.text.startswith("Your special collections day"):
                special_day = para.find("strong").text.strip()

        # The regular calendar only shows until end of March 2026, work out how many weeks that is
        weeks_total = math.floor((datetime(2026, 4, 1) - datetime.now()).days / 7)

        # The garden calendar only shows until end of November 2024, work out how many weeks that is
        garden_weeks_total = math.floor(
            (datetime(2024, 12, 1) - datetime.now()).days / 7
        )

        regular_collections, garden_collections, special_collections = [], [], []
        # Convert day text to series of dates using previous calculation
        if regular_day is not None:
            regular_collections = get_weekday_dates_in_period(
                datetime.today(),
                days_of_week.get(regular_day.capitalize()),
                amount=weeks_total,
            )
        if garden_day is not None:
            garden_collections = get_weekday_dates_in_period(
                datetime.today(),
                days_of_week.get(garden_day.capitalize()),
                amount=garden_weeks_total,
            )
        if special_day is not None:
            special_collections = get_weekday_dates_in_period(
                datetime.today(),
                days_of_week.get(special_day.capitalize()),
                amount=weeks_total,
            )

        collections = []

        # Add regular collections, and differentiate between regular and recycling bins
        for item in regular_collections:
            item_as_date = datetime.strptime(item, date_format)
            # Check if holiday (calendar only has one day that's a holiday, and it's moved to the next day)
            if is_holiday(item_as_date, Region.ENG):
                item_as_date += timedelta(days=1)
            # Use the isoweek number to separate collections based on week label.
            if (item_as_date.date().isocalendar()[1] % 2) == regular_week:
                collections.append(("Refuse (green)", item_as_date))
            else:
                collections.append(("Recycling (grey)", item_as_date))

        # Add garden collections
        for item in garden_collections:
            item_as_date = datetime.strptime(item, date_format)
            # Garden collections do not move for bank holidays
            if (item_as_date.date().isocalendar()[1] % 2) == garden_week:
                collections.append(("Garden Waste (brown)", item_as_date))

        # Add special collections
        collections += [
            ("Special Collection (bookable)", datetime.strptime(item, date_format))
            for item in special_collections
        ]

        return {
            "bins": [
                {
                    "type": item[0],
                    "collectionDate": item[1].strftime(date_format),
                }
                for item in sorted(collections, key=lambda x: x[1])
            ]
        }



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(
                "https://www.sthelens.gov.uk/article/3473/Check-your-collection-dates"
            )

            """
            accept_button = WebDriverWait(driver, timeout=30).until(
                EC.element_to_be_clickable((By.ID, "ccc-notify-accept"))
            )
            accept_button.click()
            """

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.ID, "RESIDENTCOLLECTIONDATES_PAGE1_POSTCODE")
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "RESIDENTCOLLECTIONDATES_PAGE1_FINDADDRESS_NEXT")
                )
            )
            findAddress.click()

            WebDriverWait(driver, timeout=30).until(
                EC.element_to_be_clickable(
                    (By.ID, "RESIDENTCOLLECTIONDATES_PAGE1_ADDRESS_chosen")
                )
            ).click()

            WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (
                        By.XPATH,
                        f"//ul[@id='RESIDENTCOLLECTIONDATES_PAGE1_ADDRESS-chosen-search-results']/li[starts-with(text(), '{user_paon}')]",
                    )
                )
            ).click()

            WebDriverWait(driver, timeout=30).until(
                EC.element_to_be_clickable(
                    (By.ID, "RESIDENTCOLLECTIONDATES_PAGE1_ADDRESSNEXT_NEXT")
                )
            ).click()

            # Wait for the collections table to appear
            WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "RESIDENTCOLLECTIONDATES__FIELDS_OUTER")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # Get the month rows first
            current_month = ""
            for row in soup.find_all("tr"):
                # Check if the row is a month header (contains 'th' tag)
                if row.find("th"):
                    current_month = row.find("th").get_text(strip=True)
                else:
                    # Extract the date, day, and waste types
                    columns = row.find_all("td")
                    if len(columns) >= 4:
                        day = columns[0].get_text(strip=True)
                        date = day + " " + current_month
                        waste_types = columns[3].get_text(strip=True)

                        for type in waste_types.split(" & "):
                            dict_data = {
                                "type": type,
                                "collectionDate": datetime.strptime(
                                    date,
                                    "%d %B %Y",
                                ).strftime("%d/%m/%Y"),
                            }
                            data["bins"].append(dict_data)

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber
import re


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://community.fdean.gov.uk/s/waste-collection-enquiry"

            data = {"bins": []}

            house_number = kwargs.get("paon")
            postcode = kwargs.get("postcode")
            full_address = f"{house_number}, {postcode}"
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            # If you bang in the house number (or property name) and postcode in the box it should find your property
            wait = WebDriverWait(driver, 60)
            address_entry_field = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//*[@placeholder="Search Properties..."]')
                )
            )

            address_entry_field.send_keys(str(full_address))

            address_entry_field = wait.until(
                EC.element_to_be_clickable((By.XPATH, f'//*[@title="{full_address}"]'))
            )
            address_entry_field.click()

            next_button = wait.until(
                EC.element_to_be_clickable(
                    (By.XPATH, "//lightning-button/button[contains(text(), 'Next')]")
                )
            )
            next_button.click()

            result = wait.until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        '//table[@class="slds-table slds-table_header-fixed slds-table_bordered slds-table_edit slds-table_resizable-cols"]',
                    )
                )
            )

            # Make a BS4 object
            soup = BeautifulSoup(
                result.get_attribute("innerHTML"), features="html.parser"
            )  # Wait for the 'Select your property' dropdown to appear and select the first result

            data = {"bins": []}
            today = datetime.now()
            current_year = today.year

            # Find all bin rows in the table
            rows = soup.find_all("tr", class_="slds-hint-parent")

            for row in rows:
                try:
                    bin_type_cell = row.find("th")
                    date_cell = row.find("td")

                    if not bin_type_cell or not date_cell:
                        continue

                    container_type = bin_type_cell.get("data-cell-value", "").strip()
                    raw_date_text = date_cell.get("data-cell-value", "").strip()

                    # Handle relative values like "Today" or "Tomorrow"
                    if "today" in raw_date_text.lower():
                        parsed_date = today
                    elif "tomorrow" in raw_date_text.lower():
                        parsed_date = today + timedelta(days=1)
                    else:
                        # Expected format: "Thu, 10 April"
                        # Strip any rogue characters and try parsing
                        cleaned_date = re.sub(r"[^\w\s,]", "", raw_date_text)
                        try:
                            parsed_date = datetime.strptime(cleaned_date, "%a, %d %B")
                            parsed_date = parsed_date.replace(year=current_year)
                            if parsed_date < today:
                                # Date has passed this year, must be next year
                                parsed_date = parsed_date.replace(year=current_year + 1)
                        except Exception as e:
                            print(f"Could not parse date '{cleaned_date}': {e}")
                            continue

                    formatted_date = parsed_date.strftime(date_format)
                    data["bins"].append(
                        {"type": container_type, "collectionDate": formatted_date}
                    )

                except Exception as e:
                    print(f"Error processing row: {e}")
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from datetime import datetime
from typing import Any
import requests
from ics import Calendar

from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from uk_bin_collection.uk_bin_collection.common import date_format


class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs: Any) -> dict:
        ics_url: str = kwargs.get("url")

        if not ics_url:
            raise ValueError("Missing required argument: google_calendar_ics_url")

        response = requests.get(ics_url)
        response.raise_for_status()

        calendar = Calendar(response.text)
        bindata = {"bins": []}

        for event in calendar.events:
            if not event.name or not event.begin:
                continue

            try:
                # .begin is a datetime-like object (Arrow)
                collection_date = event.begin.date().strftime(date_format)
            except Exception:
                continue

            bindata["bins"].append(
                {"type": event.name, "collectionDate": collection_date}
            )

        return bindata



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://bincollections.bromsgrove.gov.uk/BinCollections/Details?uprn={user_uprn}"

        # Make the GET request
        response = requests.get(URI)

        # Parse the HTML
        soup = BeautifulSoup(response.content, "html.parser")

        # Find each collection container
        for container in soup.find_all(class_="collection-container"):
            # Get the bin type from the heading
            bin_type = container.find(class_="heading").text.strip()

            # Get the next collection date from the caption
            next_collection = (
                container.find(class_="caption")
                .text.replace("Next collection ", "")
                .strip()
            )

            dict_data = {
                "type": bin_type,
                "collectionDate": datetime.strptime(
                    next_collection,
                    "%A, %d %B %Y",
                ).strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}
        collections = []

        curr_date = datetime.today()

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "Accept-Language": "en-GB,en;q=0.9",
            "Cache-Control": "no-cache",
            "Connection": "keep-alive",
            "Pragma": "no-cache",
            "Sec-Fetch-Dest": "document",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-Site": "none",
            "Sec-Fetch-User": "?1",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/125.0.6422.143 Safari/537.36",
            "sec-ch-ua": '"Opera GX";v="111", "Chromium";v="125", "Not.A/Brand";v="24"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
        }
        params = {
            "uprn": f"{user_uprn}",
            # 'uprn': f'100040128734',
        }
        response = requests.get(
            "https://www.cornwall.gov.uk/umbraco/surface/waste/MyCollectionDays",
            params=params,
            headers=headers,
        )

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        for item in soup.find_all("div", class_="collection text-center service"):
            bin_type = item.contents[1].text + " bin"
            try:
                collection_date = datetime.strptime(
                    item.contents[5].text, "%d %b"
                ).replace(year=curr_date.year)
            except:
                continue

            if curr_date.month == 12 and collection_date.month == 1:
                collection_date = collection_date + relativedelta(years=1)
            collections.append((bin_type, collection_date))

            ordered_data = sorted(collections, key=lambda x: x[1])
            data = {"bins": []}
            for bin in ordered_data:
                dict_data = {
                    "type": bin[0].capitalize().strip(),
                    "collectionDate": bin[1].strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        bindata = {"bins": []}

        user_uprn = user_uprn.zfill(8)

        year = datetime.today().year
        response = requests.get(
            f"https://bindayfinder.moray.gov.uk/cal_{year}_view.php",
            params={"id": user_uprn},
        )
        if response.status_code != 200:
            # fall back to known good calendar URL
            response = requests.get(
                "https://bindayfinder.moray.gov.uk/cal_2024_view.php",
                params={"id": user_uprn},
            )
        soup = BeautifulSoup(response.text, "html.parser")

        bin_types = {
            "G": "Green",
            "B": "Brown",
            "P": "Purple",
            "C": "Blue",
            "O": "Orange",
        }

        for month_container in soup.findAll("div", class_="month-container"):
            for div in month_container.findAll("div"):
                if "month-header" in div["class"]:
                    month = div.text
                elif div["class"] and div["class"][0] in ["B", "GPOC", "GBPOC"]:
                    bins = div["class"][0]
                    dom = int(div.text)
                    for i in bins:
                        dict_data = {
                            "type": bin_types.get(i),
                            "collectionDate": datetime.strptime(
                                f"{dom} {month} {year}",
                                "%d %B %Y",
                            ).strftime("%d/%m/%Y"),
                        }
                        bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get and check UPRN
        driver = None
        try:
            user_postcode = kwargs.get("postcode")
            user_paon = kwargs.get("paon")
            check_paon(user_paon)
            check_postcode(user_postcode)
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            bindata = {"bins": []}

            API_URL = "https://uhte-wrp.whitespacews.com"

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(API_URL)

            # Click Find my bin collection day button
            collectionButton = WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable((By.LINK_TEXT, "Find my bin collection day"))
            )
            collectionButton.click()

            main_content = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located((By.ID, "main-content"))
            )

            # Wait for the property number field to appear then populate it
            inputElement_number = WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (
                        By.ID,
                        "address_name_number",
                    )
                )
            )
            inputElement_number.send_keys(user_paon)

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (
                        By.ID,
                        "address_postcode",
                    )
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            continueButton = WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (
                        By.ID,
                        "Submit",
                    )
                )
            )
            continueButton.click()

            # Wait for the 'Search Results' to appear and select the first result
            property = WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (
                        By.CSS_SELECTOR,
                        "li.app-subnav__section-item a",
                        # "app-subnav__link govuk-link clicker colordarkblue fontfamilyArial fontsize12rem",
                        # "//a[starts-with(@aria-label, '{user_paon}')]",
                    )
                )
            )
            property.click()

            upcoming_scheduled_collections = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "upcoming-scheduled-collections")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            collections = []
            for collection in soup.find_all(
                "u1",
                class_="displayinlineblock justifycontentleft alignitemscenter margin0 padding0",
            ):
                date = collection.find(
                    "p", string=lambda text: text and "/" in text
                ).text.strip()  # Extract date
                service = collection.find(
                    "p", string=lambda text: text and "Collection Service" in text
                ).text.strip()  # Extract service type
                collections.append({"date": date, "service": service})

            # Print the parsed data
            for item in collections:

                dict_data = {
                    "type": item["service"],
                    "collectionDate": item["date"],
                }
                bindata["bins"].append(dict_data)

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return bindata



import time

import requests
from dateutil.relativedelta import relativedelta

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)
        user_paon = kwargs.get("paon")
        check_paon(user_paon)
        bindata = {"bins": []}

        SESSION_URL = "https://spelthorne-self.achieveservice.com/authapi/isauthenticated?uri=https%253A%252F%252Fspelthorne-self.achieveservice.com%252Fen%252FAchieveForms%252F%253Fmode%253Dfill%2526consentMessage%253Dyes%2526form_uri%253Dsandbox-publish%253A%252F%252FAF-Process-8d0df9d6-6bd0-487e-bbfe-38815dcc780d%252FAF-Stage-bce7fc80-bcd7-45f1-bf55-a76d38dbebba%252Fdefinition.json%2526process%253D1%2526process_uri%253Dsandbox-processes%253A%252F%252FAF-Process-8d0df9d6-6bd0-487e-bbfe-38815dcc780d%2526process_id%253DAF-Process-8d0df9d6-6bd0-487e-bbfe-38815dcc780d%2526noLoginPrompt%253D1&hostname=spelthorne-self.achieveservice.com&withCredentials=true"

        API_URL = "https://spelthorne-self.achieveservice.com/apibroker/runLookup"

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "Mozilla/5.0",
            "X-Requested-With": "XMLHttpRequest",
            "Referer": "https://spelthorne-self.achieveservice.com/fillform/?iframe_id=fillform-frame-1&db_id=",
        }
        s = requests.session()
        r = s.get(SESSION_URL)
        r.raise_for_status()
        session_data = r.json()
        sid = session_data["auth-session"]
        data = {
            "formValues": {
                "Property details": {
                    "postalcode": {"value": user_postcode},
                }
            }
        }
        params = {
            "id": "59b4477d6d84e",
            "repeat_against": "",
            "noRetry": "false",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }

        r = s.post(API_URL, json=data, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()

        rows_data = data["integration"]["transformed"]["rows_data"]

        for key, value in rows_data.items():

            address = value.get("display")

            if address.startswith(user_paon):
                user_uprn = value.get("value")
                break

        params = {
            "id": "5f97e6e09fedd",
            "repeat_against": "",
            "noRetry": "true",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }

        r = s.post(API_URL, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()

        rows_data = data["integration"]["transformed"]["rows_data"]["0"]

        token = rows_data.get("tokenString")

        start_date = datetime.today() + relativedelta(weeks=2)
        last2weeks = datetime.today() - relativedelta(weeks=2)

        # Format as YYYY-MM-DD
        formatted_date = start_date.strftime("%Y-%m-%d")
        last2weeks = last2weeks.strftime("%Y-%m-%d")

        data = {
            "formValues": {
                "Property details": {
                    "uprn1": {"value": user_uprn},
                    "endDate": {"value": formatted_date},
                    "last2Weeks": {"value": last2weeks},
                    "token": {"value": token},
                }
            }
        }

        params = {
            "id": "66042a164c9a5",
            "repeat_against": "",
            "noRetry": "true",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }

        r = s.post(API_URL, json=data, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()

        rows_data = data["integration"]["transformed"]["rows_data"]["0"]

        if not isinstance(rows_data, dict):
            raise ValueError("Invalid data returned from API")

        use_new = any(k.endswith("New") and v for k, v in rows_data.items())
        next_date_key = "NextDateNew" if use_new else "NextDateOld"

        for key, value in rows_data.items():
            if not (key.endswith("NextCollection") or key.endswith(next_date_key)):
                continue

            bin_type = key.split("NextCollection")[0]
            if bin_type == "Gw":
                bin_type = ["Garden Waste"]
            if bin_type == "Rec":
                bin_type = ["Recycling Bin", "Food Waste Bin"]
            if bin_type == "Ref":
                bin_type = ["Rubbish Bin", "Food Waste Bin"]

            try:
                date = datetime.strptime(value, "%Y-%m-%d").strftime("%d/%m/%Y")
            except ValueError:
                continue

            for bin in bin_type:
                dict_data = {"type": bin, "collectionDate": date}
                bindata["bins"].append(dict_data)

        return bindata



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_paon = kwargs.get("paon")
        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)
        check_paon(user_paon)
        bindata = {"bins": []}

        URI = "https://waste-api-hackney-live.ieg4.net/f806d91c-e133-43a6-ba9a-c0ae4f4cccf6/property/opensearch"

        data = {
            "Postcode": user_postcode,
        }
        headers = {"Content-Type": "application/json"}

        # Make the GET request
        response = requests.post(URI, json=data, headers=headers)

        addresses = response.json()

        for address in addresses["addressSummaries"]:
            summary = address["summary"]
            if user_paon in summary:
                systemId = address["systemId"]
        if systemId:
            URI = f"https://waste-api-hackney-live.ieg4.net/f806d91c-e133-43a6-ba9a-c0ae4f4cccf6/alloywastepages/getproperty/{systemId}"

            response = requests.get(URI)

            address = response.json()

            binIDs = address["providerSpecificFields"][
                "attributes_wasteContainersAssignableWasteContainers"
            ]
            for binID in binIDs.split(","):
                URI = f"https://waste-api-hackney-live.ieg4.net/f806d91c-e133-43a6-ba9a-c0ae4f4cccf6/alloywastepages/getbin/{binID}"
                response = requests.get(URI)
                getBin = response.json()

                bin_type = getBin["subTitle"]

                URI = f"https://waste-api-hackney-live.ieg4.net/f806d91c-e133-43a6-ba9a-c0ae4f4cccf6/alloywastepages/getcollection/{binID}"
                response = requests.get(URI)
                getcollection = response.json()

                collectionID = getcollection["scheduleCodeWorkflowIDs"][0]

                URI = f"https://waste-api-hackney-live.ieg4.net/f806d91c-e133-43a6-ba9a-c0ae4f4cccf6/alloywastepages/getworkflow/{collectionID}"
                response = requests.get(URI)
                collection_dates = response.json()

                dates = collection_dates["trigger"]["dates"]

                for date in dates:
                    parsed_datetime = datetime.strptime(
                        date, "%Y-%m-%dT%H:%M:%SZ"
                    ).strftime(date_format)

                    dict_data = {
                        "type": bin_type.strip(),
                        "collectionDate": parsed_datetime,
                    }
                    bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import time

from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            collections = []
            user_paon = kwargs.get("paon")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_paon(user_paon)
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get("https://www.westberks.gov.uk/binday")

            # Wait for the postcode field to appear then populate it
            inputElement_postcode = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.ID, "FINDYOURBINDAYS_ADDRESSLOOKUPPOSTCODE")
                )
            )
            inputElement_postcode.send_keys(user_postcode)

            # Click search button
            findAddress = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (By.ID, "FINDYOURBINDAYS_ADDRESSLOOKUPSEARCH")
                )
            )
            findAddress.click()

            WebDriverWait(driver, 10).until(
                EC.element_to_be_clickable(
                    (
                        By.XPATH,
                        ""
                        "//*[@id='FINDYOURBINDAYS_ADDRESSLOOKUPADDRESS']//option[contains(., '"
                        + user_paon
                        + "')]",
                    )
                )
            ).click()

            # Wait for the submit button to appear, then click it to get the collection dates
            WebDriverWait(driver, 30).until(
                EC.presence_of_element_located(
                    (By.XPATH, '//*[@id="FINDYOURBINDAYS_RUBBISHDATE"]/div')
                )
            )
            time.sleep(2)

            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            rubbish_div = soup.find(
                "div", {"id": "FINDYOURBINDAYS_RUBBISHDATE_OUTERDIV"}
            )
            rubbish_date = rubbish_div.find_all("div")[2]
            if rubbish_date.text == "Today":
                rubbish_date = datetime.now()
            else:
                rubbish_date = datetime.strptime(
                    rubbish_date.text,
                    "%A %d %B",
                ).replace(year=datetime.now().year)

            recycling_div = soup.find(
                "div", {"id": "FINDYOURBINDAYS_RECYCLINGDATE_OUTERDIV"}
            )
            recycling_date = recycling_div.find_all("div")[2]
            if recycling_date.text == "Today":
                recycling_date = datetime.now()
            else:
                recycling_date = datetime.strptime(
                    recycling_date.text,
                    "%A %d %B",
                ).replace(year=datetime.now().year)

            food_div = soup.find(
                "div", {"id": "FINDYOURBINDAYS_FOODWASTEDATE_OUTERDIV"}
            )
            food_date = food_div.find_all("div")[2]
            if food_date.text == "Today":
                food_date = datetime.now()
            else:
                food_date = datetime.strptime(
                    food_date.text,
                    "%A %d %B",
                ).replace(year=datetime.now().year)

            if datetime.now().month == 12 and rubbish_date.month == 1:
                rubbish_date = rubbish_date + relativedelta(years=1)
            if datetime.now().month == 12 and recycling_date.month == 1:
                recycling_date = recycling_date + relativedelta(years=1)
            if datetime.now().month == 12 and food_date.month == 1:
                food_date = food_date + relativedelta(years=1)

            collections.append(("Rubbish bin", rubbish_date))
            collections.append(("Recycling bin", recycling_date))
            collections.append(("Food waste bin", food_date))

            ordered_data = sorted(collections, key=lambda x: x[1])
            for item in ordered_data:
                dict_data = {
                    "type": item[0].capitalize(),
                    "collectionDate": item[1].strftime(date_format),
                }
                data["bins"].append(dict_data)

            print()
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = "https://selfserve.worcester.gov.uk/wccroundlookup/HandleSearchScreen"

        post_data = {
            "alAddrsel": user_uprn,
        }

        headers = {
            "referer": "https://selfserve.worcester.gov.uk/wccroundlookup/HandleSearchScreen",
            "content-type": "application/x-www-form-urlencoded",
        }

        response = requests.post(URI, data=post_data, headers=headers, verify=False)

        soup = BeautifulSoup(response.content, "html.parser")
        rows = soup.select("table.table tbody tr")

        for row in rows:
            bin_type = row.select_one("td:nth-of-type(2)").text.strip()
            collection_date = row.select_one("td:nth-of-type(3) strong").text.strip()

            if collection_date == "Not applicable":
                continue

            dict_data = {
                "type": bin_type,
                "collectionDate": datetime.strptime(
                    collection_date,
                    "%A %d/%m/%Y",
                ).strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://secure.derby.gov.uk/binday/Binday?search.PremisesId={user_uprn}"

        # Make the GET request
        session = requests.Session()
        response = session.get(URI)

        soup = BeautifulSoup(response.content, "html.parser")

        # Find all divs with class "binresult" which contain the bin collection information
        bin_results = soup.find_all("div", class_="binresult")

        # Loop through each bin result to extract date and bin type
        for result in bin_results:
            # Find the collection date
            date_text = result.find("p").strong.get_text(strip=True)

            # Find the bin type by looking at the 'alt' attribute of the img tag
            bin_type = result.find("img")["alt"]

            if bin_type != "No bins":
                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(
                        date_text,
                        "%A, %d %B %Y:",
                    ).strftime(date_format),
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



#!/usr/bin/env python3

# This script pulls (in one hit) the data from
# Newcastle City Council Bins Data
from datetime import datetime

from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import date_format
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page, **kwargs) -> None:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        for element in soup.find_all("strong"):
            collectionInfo = ""
            # Domestic Waste is formatted differently to other bins
            if "Green Bin (Domestic Waste) details:" in str(element):
                if element.next_sibling.find("br"):
                    collectionInfo = element.next_sibling.find("br").next_element
            elif "Next collection" in str(
                element.next_sibling.next_sibling.next_sibling.next_sibling
            ):
                collectionInfo = (
                    element.next_sibling.next_sibling.next_sibling.next_sibling
                )

            if collectionInfo != "" and collectionInfo != "Next collection : n/a":
                bin_type = str(element)[
                    str(element).find("(") + 1 : str(element).find(")")
                ]
                collectionDate = str(
                    datetime.strptime(
                        str(collectionInfo).replace("Next collection : ", ""),
                        "%d-%b-%Y",
                    )
                    .date()
                    .strftime(date_format)
                )

                dict_data = {"type": bin_type, "collectionDate": collectionDate}

                data["bins"].append(dict_data)

        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://my.oadby-wigston.gov.uk/location?put=ow{user_uprn}&rememberme=0&redirect=%2F"

        # Make the GET request
        response = requests.get(URI)

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        # Find the collection list
        collection_list = soup.find("ul", class_="refuse")

        current_year = datetime.now().year
        next_year = current_year + 1

        # Loop through each collection item
        for li in collection_list.find_all("li"):
            date_text = li.find("strong", class_="date").text.strip()
            bin_type = li.find("a").text  # Get the class for bin type

            # Parse the date
            if date_text == "Today":
                collection_date = datetime.now()
            else:
                try:
                    collection_date = datetime.strptime(date_text, "%A %d %b")
                except:
                    continue

            if (datetime.now().month == 12) and (collection_date.month == 1):
                collection_date = collection_date.replace(year=next_year)
            else:
                collection_date = collection_date.replace(year=current_year)

            dict_data = {
                "type": bin_type,
                "collectionDate": collection_date.strftime(date_format),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata
